var Client = require("mysql").Client;
var DBClient = function (client) {
	this._client = client;
};
DBClient.prototype.createCollection = function (collection, fields, assocs) {
	var _table = collection.toLowerCase();
	var _query = "CREATE TABLE IF NOT EXISTS `%table` (%values) ENGINE = INNODB CHARACTER SET utf8 COLLATE utf8_general_ci", _fields = [], _indexes = [];
	_query = _query.replace("%table", _table);
	
	var field_names = [];
	var add_id = (!fields._params || fields._params.indexOf("no-id") == -1);
	var unique_record = (fields._params && fields._params.indexOf("unique-record") != -1);
	
	if (add_id) {
		_fields.push("`id` BIGINT(10) UNSIGNED NOT NULL AUTO_INCREMENT");
	}
	for (k in fields) {
		if (k == "_params") {
			continue;
		}

		var field = "`" + k + "`";
		
		switch (fields[k].type) {
			case "enum":
				field += " ENUM ('" + fields[k].values.join("', '") + "')";
				break;
			case "struct":
			case "text":	field += " TEXT"; break;
			case "num":
			case "number":
			case "int":
			case "integer":	field += " INT"; break;
			case "float":	field += " FLOAT"; break;
			case "bool":
			case "boolean":	field += " TINYINT(1)"; break;
			case "date":	field += " DATETIME"; break;
			case "data":	field += " BLOB"; break;
			default:
				field += " VARCHAR(255)";
		}
		
		field_names.push(k);
		_fields.push(field + " NOT NULL");
	}
	
	for (var i = 0; i < assocs.length; i++) {
		switch (assocs[i].type) {
			case "one":
				_fields.push("`" + assocs[i].field + "_id` BIGINT(10) UNSIGNED NOT NULL");
				_indexes.push(assocs[i].field + "_id");
				field_names.push(assocs[i].field + "_id");
				break;
			case "many":
				this.createCollection(_table + "_" + assocs[i].field, {
					"_params": [ "unique-record", "no-id" ]
				}, [{
					"field"	: _table,
					"type"	: "one",
					"entity": this
				}, {
					"field"	: assocs[i].name || assocs[i].field,
					"type"	: "one",
					"entity": assocs[i].entity
				}]);
				break;
		}
	}
	
	if (add_id) {
		_fields.push("PRIMARY KEY (`id`)");
	}
	
	if (unique_record) {
		_fields.push("PRIMARY KEY (`" + field_names.join("`, `") + "`)");
	} else {
		for (var i = 0; i < _indexes.length; i++) {
			_fields.push("INDEX (`" + _indexes[i] + "`)");
		}
	}
	
	_query = _query.replace("%values", _fields.join(", "));
	
	this._client.query(_query, function (err, info) {
		/*
		console.log(err);
		console.log(info);
		console.log("collection synced");
		*/
	});
};
DBClient.prototype.selectRecords = function (collection, config) {
	var _table = collection.toLowerCase(collection);
	var _query = "SELECT * FROM `" + _table + "`";

	config = config || {};

	if (config.join2) {
		_query = "SELECT t1.*, t2.* "
		       + "FROM (`" + _table + "` t1, `" + config.join2.collection + "` t2) "
		       + "JOIN `" + config.join2.join + "` j "
		       + "ON t1.id = j.`" + config.join2.collectionField + "_id` "
		       + "AND t2.id = j.`" + config.join2.joinField + "_id`";
	}

	if (config.conditions) _query = this._addQueryConditions(_query, config.conditions);
	if (config.order) _query = this._addQueryOrder(_query, config.order);
	if (config.limit) _query = this._addQueryLimit(_query, config.limit, config.skip);

	// console.log(_query);

	this._client.query(_query, function (err, info) {
		if (err) {
			config.callback(err);
			return;
		}

		if (config.join2) {
			// split data from namespaces
			for (var i = 0; i < info.length; i++) {
				var t1 = {}, t2 = {};

				for (k in info[i]) {
					if (!info[i].hasOwnProperty(k)) continue;

					if (k.substr(0, _table.length + 1) == _table + ".") {
						t1[k.substr(_table.length + 1)] = info[i][k];
					} else {
						t2[k.substr(config.join2.collection.length + 1)] = info[i][k];
					}
				}

				info[i] = {};
				info[i][_table] = t1;
				info[i][config.join2.collection] = t2;
			}
		}

		config.callback(null, info);
	});
};
DBClient.prototype.clearRecords = function (collection, config, callback) {
	var _table = collection.toLowerCase(collection);
	var _query = "DELETE FROM `" + _table + "`";

	config = config || {};

	if (config.conditions) _query = this._addQueryConditions(_query, config.conditions);
	if (config.order) _query = this._addQueryOrder(_query, config.order);
	if (config.limit) _query = this._addQueryLimit(_query, config.limit, config.skip);
	
	//console.log(_query);

	this._client.query(_query, function (err, info) {
		if (err) {
			config.callback(err);
			return;
		}
		
		config.callback(null, info);
	});
};
DBClient.prototype.saveRecord = function (collection, data, callback) {
	if (parseInt(data.id) > 0) {
		var id = data.id;
		delete data.id;
		
		this._updateRecord(collection, data, id, callback);
	} else {
		this._insertRecord(collection, data, callback);
	}
};
DBClient.prototype._insertRecord = function (collection, data, callback) {
	var _table = collection.toLowerCase();
	var _query = "INSERT INTO `" + _table + "` (%fields) VALUES (%values)", _fields = [], _values = [];
	
	for (k in data) {
		if (!data.hasOwnProperty(k)) continue;
		
		_fields.push("`" + k + "`");

		switch (typeof data[k]) {
			case "number":
				_values.push(data[k]);
				break;
			case "boolean":
				_values.push(data[k] ? 1 : 0);
				break;
			case "undefined":
				_fields.pop();
				break;
			default:
				_values.push("'" + data[k].replace("'", "\\'") + "'");
		}
	}
	
	_query = _query.replace("%fields", _fields.join(", "));
	_query = _query.replace("%values", _values.join(", "));

	// console.log(_query);
	
	this._client.query(_query, function (err, info) {
		if (err) {
			callback(err);
			return;
		}
		
		callback(null, info.insertId);
	});
};
DBClient.prototype._updateRecord = function (collection, data, id, callback) {
	var _table = collection.toLowerCase();
	var _query = "UPDATE `" + _table + "` SET %values WHERE `id`=" + id, _values = [];
	
	for (k in data) {
		if (!data.hasOwnProperty(k)) continue;

		switch (typeof data[k]) {
			case "number":
				_values.push("`" + k + "`=" + data[k]);
				break;
			case "boolean":
				_values.push("`" + k + "`=" + (data[k] ? 1 : 0));
				break;
			default:
				if (data[k] === null) {
					_values.push("`" + k + "`=NULL");
				} else {
					if (typeof data[k] == "object") console.dir(data[k]);
					_values.push("`" + k + "`='" + data[k].replace("'", "\\'") + "'");
				}
		}
	}
	
	_query = _query.replace("%values", _values.join(", "));

	// console.log(_query);
	
	this._client.query(_query, function (err, info) {
		if (err) {
			callback(err);
			return;
		}
		
		callback(null);
	});
};
DBClient.prototype._addQueryConditions = function (query, conditions) {
	var _conditions = [];

	for (k in conditions) {
		if (!conditions.hasOwnProperty(k)) continue;
		
		switch (typeof conditions[k]) {
			case "number":
				_conditions.push("`" + k + "`=" + conditions[k]);
				break;
			case "boolean":
				_conditions.push("`" + k + "`=" + (conditions[k] ? 1 : 0));
				break;
			default:
				if (conditions[k].forEach) {
					_conditions.push("`" + k + "` IN ('" + conditions[k].join("', '") + "')");
				} else {
					_conditions.push("`" + k + "`='" + conditions[k].replace("'", "\\'") + "'");
				}
		}
	}
	
	return query + " WHERE " + _conditions.join(" AND ");
};
DBClient.prototype._addQueryOrder = function (query, order) {
	if (!order.match(/\s(asc|desc)$/i)) {
		order += " asc";
	}
	
	return query + " ORDER BY " + order.replace(/\w+/g, function (word) {
		if ([ "ASC", "DESC" ].indexOf(word.toUpperCase()) != -1)
			return word.toUpperCase();
		return "`" + word + "`";
	});
};
DBClient.prototype._addQueryLimit = function (query, limit, skip) {
	if (skip) {
		return query + " LIMIT " + skip + ", " + limit;
	}
	return query + " LIMIT " + limit;
};

exports.connect = function (options, callback) {
	var client = new Client();
	var opts = {
		"host"		: "localhost",
		"port"		: 3306,
		"user"		: "root",
		"password"	: "",
		"database"	: "test"
	};
	
	if (options.auth) {
		var p;
		if ((p = options.auth.indexOf(":")) != -1) {
			options.user = options.auth.substr(0, p);
			options.password = options.auth.substr(p + 1);
		} else {
			options.user = options.auth;
		}
	}
	if (options.pathname) {
		options.database = options.pathname.substr(1);
	}
	if (options.hostname) {
		options.host = options.hostname;
	}
	
	for (k in options) {
		if (opts.hasOwnProperty(k)) {
			opts[k] = options[k];
		}
	}
	for (k in opts) {
		if (opts.hasOwnProperty(k)) {
			client[k] = opts[k];
		}
	}

	client.connect(function (err) {
		if (err && err.number) {
			callback(false, { "number": err.number, "message": err.message });
		} else {
			callback(true, new DBClient(client));
		}
	});
};
